/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('deployment-summary-classic', function () {

  juju.components.DeploymentSummaryClassic = React.createClass({

    propTypes: {
      autoPlaceDefault: React.PropTypes.bool.isRequired,
      autoPlaceUnits: React.PropTypes.func.isRequired,
      changeDescriptions: React.PropTypes.array.isRequired,
      changeState: React.PropTypes.func.isRequired,
      ecsClear: React.PropTypes.func.isRequired,
      ecsCommit: React.PropTypes.func.isRequired,
      getUnplacedUnitCount: React.PropTypes.func.isRequired
    },

    /**
      Get the current state of the component.
       @method getInitialState
      @returns {String} The current state.
    */
    getInitialState: function getInitialState() {
      // Setting a default state object.
      return {
        autoPlace: this.props.autoPlaceDefault
      };
    },

    /**
      Close the summary.
       @method _close
    */
    _close: function _close() {
      this.props.changeState({
        sectionC: {
          component: null,
          metadata: {}
        }
      });
    },

    /**
      Handles calling to clear the ecs and then closing the deployment
      summary.
       @method _handleClear
    */
    _handleClear: function _handleClear() {
      this.props.ecsClear();
      this._close();
    },

    /**
      Handle committing when the deploy button in the summary is clicked.
       @method _handleDeploy
    */
    _handleDeploy: function _handleDeploy() {
      if (this.state.autoPlace) {
        this.props.autoPlaceUnits();
      }
      // The env is already bound to ecsCommit in app.js.
      this.props.ecsCommit();
      this._close();
    },

    /**
      Handle navigating to the machine view.
       @method _handleViewMachinesClick
    */
    _handleViewMachinesClick: function _handleViewMachinesClick() {
      this.props.changeState({
        sectionB: {
          component: 'machine',
          metadata: {}
        },
        sectionC: {
          component: null,
          metadata: {}
        }
      });
    },

    /**
      Handle changes to the placement radio buttons.
       @method _handlePlacementChange
      @param {Object} e The click event.
    */
    _handlePlacementChange: function _handlePlacementChange(e) {
      this.setState({
        autoPlace: e.currentTarget.getAttribute('data-placement') === 'placed'
      });
    },

    /**
      Generate the list of change items.
       @method _generateChangeItems
      @returns {Array} The collection of changes.
    */
    _generateChangeItems: function _generateChangeItems() {
      var changeList = this.props.changeDescriptions;
      var changes = [];
      changeList.forEach(function (change, i) {
        changes.push(React.createElement(juju.components.DeploymentSummaryChangeItemClassic, {
          key: i,
          change: change }));
      }, this);
      return changes;
    },

    render: function render() {
      var listHeaderClassName = 'deployment-summary-change-item-classic ' + 'deployment-summary-classic__list-header';
      return React.createElement(
        juju.components.Panel,
        {
          instanceName: 'white-box',
          visible: true },
        React.createElement(
          'div',
          { className: 'deployment-summary-classic' },
          React.createElement(
            'div',
            { className: 'deployment-summary-classic__header' },
            React.createElement(
              'span',
              { className: 'deployment-summary-classic__close',
                tabIndex: '0', role: 'button',
                onClick: this._close },
              React.createElement(juju.components.SvgIcon, { name: 'close_16',
                size: '16' })
            ),
            React.createElement(
              'h2',
              { className: 'deployment-summary-classic__title' },
              'Deployment summary'
            ),
            React.createElement(juju.components.DeploymentSummaryPlacementClassic, {
              handleViewMachinesClick: this._handleViewMachinesClick,
              handlePlacementChange: this._handlePlacementChange,
              autoPlace: this.props.autoPlaceDefault,
              getUnplacedUnitCount: this.props.getUnplacedUnitCount })
          ),
          React.createElement(
            'div',
            { className: 'deployment-summary-classic__content' },
            React.createElement(
              'ul',
              { className: 'deployment-summary-classic__list' },
              React.createElement(
                'li',
                { className: listHeaderClassName },
                React.createElement(
                  'span',
                  { className: 'deployment-summary-change-item-classic__change' },
                  'Change'
                ),
                React.createElement(
                  'span',
                  { className: 'deployment-summary-change-item-classic__time' },
                  'Time'
                )
              ),
              this._generateChangeItems()
            )
          ),
          React.createElement(
            'div',
            { className: 'deployment-summary-classic__footer' },
            React.createElement(juju.components.GenericButton, {
              type: 'inline-neutral',
              action: this._handleClear,
              title: 'Clear changes' }),
            React.createElement(juju.components.GenericButton, {
              action: this._handleDeploy,
              type: 'inline-positive',
              title: 'Deploy' })
          )
        )
      );
    }
  });
}, '0.1.0', { requires: ['deployment-summary-change-item-classic', 'deployment-summary-placement-classic', 'generic-button', 'panel-component', 'svg-icon'] });