/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('deployment-choose-cloud', function () {

  juju.components.DeploymentChooseCloud = React.createClass({

    propTypes: {
      changeCounts: React.PropTypes.object.isRequired,
      changeState: React.PropTypes.func.isRequired,
      jem: React.PropTypes.object.isRequired,
      pluralize: React.PropTypes.func.isRequired,
      services: React.PropTypes.array.isRequired,
      setDeploymentInfo: React.PropTypes.func.isRequired,
      user: React.PropTypes.object.isRequired
    },

    getInitialState: function getInitialState() {
      return {
        credentials: []
      };
    },

    componentWillMount: function componentWillMount() {
      var _this = this;

      this.props.jem.listTemplates(function (error, credentials) {
        // XXX kadams54: This is basic error handling for the initial
        // implementation. It should be replaced with an error message for the
        // user in subsequent UI polish work.
        if (error) {
          console.error('Unable to list templates', error);
          return;
        }
        _this.setState({ credentials: credentials });
      });
    },

    /**
      Generate the list of credentials.
       @method _generateCredentials
    */
    _generateCredentials: function _generateCredentials() {
      var _this2 = this;

      var credentials = this.state.credentials;
      if (!credentials || credentials.length === 0) {
        return;
      }
      var components = [];
      credentials.forEach(function (credential, i) {
        var className = classNames('deployment-choose-cloud__cloud-option', 'deployment-choose-cloud__cloud-option--credential', 'six-col', { 'last-col': i % 2 === 1 });
        components.push(React.createElement(
          'li',
          { className: className,
            key: credential.path,
            onClick: _this2._handleCredentialClick.bind(_this2, credential.path) },
          React.createElement(
            'span',
            { className: 'deployment-choose-cloud__cloud-option-title' },
            credential.path
          )
        ));
      });
      return React.createElement(
        'div',
        null,
        React.createElement(
          'h3',
          { className: 'deployment-panel__section-title twelve-col' },
          'Your cloud credentials'
        ),
        React.createElement(
          'ul',
          { className: 'deployment-choose-cloud__list twelve-col' },
          components
        )
      );
    },

    /**
      Generate a list of cloud options.
       @method _generateChangeItems
      @returns {Array} The collection of changes.
    */
    _generateOptions: function _generateOptions() {
      var options = [{
        id: 'aws',
        image: 'aws.png'
      }];
      var components = [];
      options.forEach(function (option, i) {
        var lastCol = i % 2 === 1 ? 'last-col' : '';
        var className = 'deployment-choose-cloud__cloud-option six-col ' + lastCol;
        var src = 'juju-ui/assets/images/non-sprites/' + option.image;
        components.push(React.createElement(
          'li',
          { className: className,
            key: option.id,
            onClick: this._handleCloudClick.bind(this, option.id) },
          React.createElement('img', { alt: option.id,
            src: src })
        ));
      }, this);
      return components;
    },

    /**
      Handling clicking on a cloud option.
       @method _handleCloudClick
    */
    _handleCloudClick: function _handleCloudClick(id) {
      this.props.changeState({
        sectionC: {
          component: 'deploy',
          metadata: {
            activeComponent: 'add-credentials'
          }
        }
      });
    },

    /**
      Handling clicking on an existing credential option.
       @method _handleCredentialClick
    */
    _handleCredentialClick: function _handleCredentialClick(id) {
      this.props.setDeploymentInfo('templateName', id);
      this.props.changeState({
        sectionC: {
          component: 'deploy',
          metadata: {
            activeComponent: 'summary'
          }
        }
      });
    },

    /**
      Generate the onboarding if there are no credentials.
       @method _generateOnboarding
    */
    _generateOnboarding: function _generateOnboarding() {
      var credentials = this.state.credentials;
      if (credentials && credentials.length > 0) {
        return;
      }
      return React.createElement(
        'div',
        { className: 'deployment-panel__notice twelve-col' },
        React.createElement(juju.components.SvgIcon, {
          name: 'general-action-blue',
          size: '16' }),
        'Add a public cloud credential, and we can save it as an option for later use'
      );
    },

    /**
      Generate the list of services.
       @method _generateServices
    */
    _generateServices: function _generateServices() {
      var services = this.props.services;
      if (services.length === 0) {
        return;
      }
      var components = [];
      services.forEach(function (service, i) {
        var className = classNames('two-col', { 'last-col': i % 3 === 1 });
        var name = service.get('name');
        components.push(React.createElement(
          'li',
          { className: className,
            key: service.get('id') },
          React.createElement('img', { alt: name,
            className: 'deployment-choose-cloud__services-icon',
            src: service.get('icon') }),
          name
        ));
      });
      return React.createElement(
        'ul',
        { className: 'deployment-choose-cloud__services twelve-col' },
        components
      );
    },

    render: function render() {
      var username = this.props.user && this.props.user.usernameDisplay;
      var title = 'Welcome back, ' + username;
      var pluralize = this.props.pluralize;
      var changeCounts = this.props.changeCounts;
      var serviceCount = changeCounts['_deploy'] || 0;
      var machineCount = changeCounts['_addMachines'] || 0;
      return React.createElement(
        'div',
        { className: 'deployment-panel__child' },
        React.createElement(
          juju.components.DeploymentPanelContent,
          {
            title: title },
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(
              'h3',
              { className: 'deployment-panel__section-title' },
              'Deployment summary ',
              React.createElement(
                'span',
                { className: 'deployment-panel__section-title-count' },
                '(',
                serviceCount,
                ' ',
                pluralize('service', serviceCount),
                ', ',
                machineCount,
                ' ',
                pluralize('machine', machineCount),
                ')'
              )
            ),
            this._generateServices()
          ),
          React.createElement(
            'div',
            { className: 'six-col last-col' },
            React.createElement(
              'h3',
              { className: 'deployment-panel__section-title' },
              'Unplaced units'
            ),
            React.createElement('div', { className: 'deployment-panel__box' })
          ),
          this._generateCredentials(),
          this._generateOnboarding(),
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title twelve-col' },
            'Public clouds'
          ),
          React.createElement(
            'ul',
            { className: 'deployment-choose-cloud__list twelve-col' },
            this._generateOptions()
          ),
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title twelve-col' },
            'Get credentials by signing up with your favoured public cloud'
          ),
          React.createElement(
            'ul',
            { className: 'deployment-choose-cloud__list twelve-col' },
            React.createElement(
              'li',
              null,
              React.createElement(
                'a',
                { className: 'deployment-choose-cloud__link',
                  href: 'https://cloud.google.com/compute/',
                  target: '_blank' },
                'Google Compute Engine ›'
              )
            ),
            React.createElement(
              'li',
              null,
              React.createElement(
                'a',
                { className: 'deployment-choose-cloud__link',
                  href: 'https://azure.microsoft.com/',
                  target: '_blank' },
                'Windows Azure ›'
              )
            ),
            React.createElement(
              'li',
              null,
              React.createElement(
                'a',
                { className: 'deployment-choose-cloud__link',
                  href: 'https://aws.amazon.com/',
                  target: '_blank' },
                'Amazon Web Services ›'
              )
            )
          )
        )
      );
    }
  });
}, '0.1.0', { requires: ['deployment-panel-content', 'svg-icon'] });